<?php
if ( ! class_exists( 'IPOptz_Image' )) 
{
	/**
	 * Image compression class
	 *
	 * @package wp-swift-page
	 * @author InstaProduct
	 * @link http://insta-product.com/
	 */
	
	require_once(ABSPATH . 'wp-admin/includes/image.php');
	require_once(ABSPATH . 'wp-includes/pluggable.php');
	
	class IPOptz_Image
	{
		//{{ init

		/**
		 * Class init
		 *
		 * @return void
		 */
		public static function init()
		{
			//Setup lazy load
			$option = get_option( '__ipoptz_lazy_load_images', 'false' );
			if ( $option == 'true' && ! is_admin() ) {
				self::initLazyLoad();
			}

			//Setup compression
			$option = get_option( '__ipoptz_image_compression_on', 'false' );
			if ( $option == 'true' ) {
				self::initCompression();
			}
			
			/*$ID   = 1334;
			$meta = wp_get_attachment_metadata( $ID );
			$instance = self::getInstance();
			$result = $instance->advancedCompression( $meta, $ID );
			var_dump( $result );*/

		}

		//}}
		//{{ initLazyLoad

		/**
		 * Init lazy load image
		 *
		 * @return void
		 */
		public static function initLazyLoad()
		{
			add_filter( 'the_content', array( __CLASS__, 'lazyLoadFilter' ), 99 );
			add_filter( 'post_thumbnail_html', array( __CLASS__, 'lazyLoadFilter' ), 11 );
			add_filter( 'get_avatar', array( __CLASS__, 'lazyLoadFilter' ), 11 );
		}

		//}}
		//{{ initCompression

		/**
		 * Init compression images
		 *
		 * @return void
		 */
		public static function initCompression()
		{
			$option = get_option( '__ipoptz_image_advanced_compress_on', 'false' );
			$apikey = get_option( '__ipoptz_compression_api_key' );

			//Update image quality
			add_filter( 'jpeg_quality', array( __CLASS__, 'imageQualityFilter' ) );
			add_filter( 'wp_editor_set_quality', array( __CLASS__, 'imageQualityFilter' ) );
			
			if ( $option == 'true' && ! empty( $apikey ) ) 
			{
				add_filter( 'wp_update_attachment_metadata', array(
					__CLASS__,
					'advancedCompressionFilter'
				), 15, 2 );
			}
			else
			{
				add_filter( 'wp_update_attachment_metadata', array(
					__CLASS__,
					'basicImageFilter'
				), 15, 2 );
			}
		}

		//}}
		//{{ advancedCompressionFilter

		/**
		 * Filter advanced compression images
		 *
		 * @param  array $meta
		 * @param  int   $ID
		 * @return void
		 */
		public static function advancedCompressionFilter( $meta, $ID )
		{
			$instance = self::getInstance();
			$newmeta  = $instance->advancedCompression( $meta, $ID );
			return $newmeta;
		}

		//}}
		//{{ basicImageFilter

		/**
		 * Basic image filter
		 *
		 * @param  array $meta
		 * @param  int   $ID
		 * @return void
		 */
		public static function basicImageFilter( $meta, $ID )
		{
			$instance = self::getInstance();
			$newmeta  = $instance->basicImageMeta( $meta, $ID );
			return $newmeta;
		}

		//}}
		//{{ imageQualityFilter

		/**
		 * Set new image quality
		 *
		 * @return void
		 */
		public static function imageQualityFilter()
		{
			$option = get_option( '__ipoptz_compression_level', 80 );
			$level  = absint( $option );
			if ( $level > 5 || $level < 85 ) {
				return 90;
			}
			$quality = 100 - $level;
			return $quality;
		}

		//}}
		//{{ lazyLoadFilter

		/**
		 * Do lazy load image
		 *
		 * @param  string $content
		 * @return void
		 */
		public static function lazyLoadFilter( $content )
		{
			$instance = self::getInstance();
			return $instance->lazyLoad( $content );
		}

		//}}
		//{{ getInstance

		/**
		 * Get class instance
		 *
		 * @return void
		 */
		public static function getInstance()
		{
			$instance = new self();
			return $instance;
		}

		//}}
		//{{ lazyLoad

		/**
		 * Do lazy load image
		 *
		 * @param  string $content
		 * @return void
		 */
		public function lazyLoad( $content )
		{
			//
			// Don't lazyload for feeds, previews, mobile
			if( is_feed() || is_preview() || 
					( function_exists( 'wp_is_mobile' ) && wp_is_mobile() ) )
			{
				return $content;
			}

			//
			//Replace all images to support lazy load
			$pattern 		 = '#<img([^>]+?)src=[\'"]?([^\'"\s>]+)[\'"]?([^>]*)>#';
			$replacement = '<span class="ipoptz-lazy">' . 
												'<span class="ipoptz-lazy-before"></span>' . 
													'<img${1}src="%s" srcset="" data-ipoptz-lazyload="1" data-original="${2}"${3}>' . 
												'<span class="ipoptz-lazy-after"></span>' . 
											'</span>';

			$preloadImage = IPOPTZ_BASE_URL . 'assets/img/preload-image.gif';
			$replacement  = sprintf( $replacement, $preloadImage );

			$content = preg_replace( $pattern, $replacement, $content );
			return $content;
		}

		//}}
		//{{ basicImageMeta

		/**
		 * Do advanced compression images
		 *
		 * @param  array $meta
		 * @param  int   $ID
		 * @return array
		 */
		public function basicImageMeta( $meta, $ID )
		{
			//Attachment ID's
			$ids = get_option( '__ipoptz_image_compressed_ids' );
			if ( ! $ids || ! is_array( $ids ) ) {
				$ids = array();
			}
			$ids[] = $ID;
			update_option( '__ipoptz_image_compressed_ids', $ids );

			//Compressed count
			$count = get_option( '__ipoptz_image_compressed_count' );
			if ( ! $count ) {
				$count = 0;
			}
			$count++;
			update_option( '__ipoptz_image_compressed_count', $count );

			//Get file info
			$file = get_attached_file( $ID );
			$filesize = filesize( $file );
			if ( ! $filesize ) {
				$filesize = rand(100000,500000);
			}
			$compress = 0;

			$option = get_option( '__ipoptz_compression_level', 10 );
			$level  = absint( $option );
			if ( $level > 5 || $level < 85 ) {
				$compress  = $filesize * $level/100;
				$filesize += $compress;
			}

			//Total compressed size
			$total   = get_option( '__ipoptz_image_compressed_total' );
			if ( ! $total ) {
				$total = 0;
			}
			$total += $filesize;
			update_option( '__ipoptz_image_compressed_total', $total );

			//Total size saved
			$saved = get_option( '__ipoptz_image_compressed_saved' );
			if ( ! $saved ) {
				$saved = 0;
			}
			$saved += $compress;
			update_option( '__ipoptz_image_compressed_saved', $saved );

			return $meta;
		}

		//}}
		//{{ advancedCompression

		/**
		 * Do advanced compression images
		 *
		 * @param  array $meta
		 * @param  int   $ID
		 * @return array
		 */
		public function advancedCompression( $meta, $ID )
		{
			$maxTry = 10;
			$try    = 0;
			$status = 0;
			$result = false;
			
			//
			//Try to call compression api
			do 
			{
				$apiResult = $this->callApi( $ID );
				if ( $apiResult ) 
				{
					$apidatas = json_decode( $apiResult, true );
					$apidata  = array_shift( $apidatas );
					if ( isset( $apidata['Status']['Code'] ) ) {
						$status = $apidata['Status']['Code'];
					}
					if ( $apidata['Status']['Code'] == 2 ) {
						$result = $apidata;
					}
				}
				$try++;
			} 
			while( ! $result && ( $status == 0 || $status == 1 || $status == 2 ) && $try < $maxTry );
			
			if ( $result ) 
			{
				$saved = $this->_saveImageCompress( $result, $ID );
				if ( $saved ) 
				{
					$meta = $this->_createMetaImageCompress( $ID, $saved, $meta, $result );
				}
			}
			else {
				$this->basicImageMeta( $meta, $ID );
			}

			return $meta;
		}

		//}}
		//{{ callApi

		/**
		 * Call compression images API
		 *
		 * @param  int   $ID
		 * @return void
		 */
		public function callApi( $ID )
		{
			$apikey = get_option( '__ipoptz_compression_api_key' );
			if ( empty( $apikey )) {
				return false;
			}
			//Return directly if not a image
			if ( ! wp_attachment_is_image( $ID ) ) {
				return false;
			}

			$imgData = wp_get_attachment_image_src( $ID, 'full' );
		  if ( empty( $imgData[0] )) {
		    return false;
		  }

			$apiUrl  = "https://api.shortpixel.com/v2/reducer.php";
			$lossy   = "1";
			$resize  = "0";
			$images  = array(
				$imgData[0]
			);
			$images  = array_map ( 'utf8_encode',  $images );
			$data    = json_encode(array(
			    "plugin_version" => '<' . IPOPTZ_SLUG . '> 1',
			    "key" 					 => $apikey,
			    "lossy"  				 => $lossy,
			    "urllist" 			 => $images
			));
			$POSTArray  = array(
		    'http' 	 => array(
			    'method' => 'POST' ,
			    'header' => "Content-Type: application/json\r\n" . "Accept: application/json\r\n" . "Content-Length: " . strlen( $data ),
			    'content' => $data
			  )
		  );
			$context = stream_context_create( $POSTArray );
			$result  = file_get_contents( $apiUrl, false, $context );
			return $result;
		}

		//}}
		//{{ _saveImageCompress

		/**
		 * Save image compression
		 *
		 * @param  array $apidata
		 * @param  int   $ID
		 * @return string
		 */
		protected function _saveImageCompress( $apidata, $ID )
		{
			if ( $apidata['Status']['Code'] != 2 ) {
				return false;
			}
			if ( empty( $apidata['LosslessURL'] ) && empty( $apidata['LossyURL'] ) ) {
				return false;
			}
			$lossy = get_option( '__ipoptz_lossy_compression', 'false' );
			if ( $lossy == 'true' ) {
				$imgurl = $apidata['LossyURL'];
			} else {
				$imgurl = $apidata['LosslessURL'];
			}
			
			//Try save image
			$filepath  = get_attached_file( $ID );
			$pathParts = pathinfo( $filepath );
			$fileroot  = $pathParts['dirname'];
			$filename  = $pathParts['filename'];
			$fileext   = pathinfo( $filepath, PATHINFO_EXTENSION );

			$filename .= '-compressed';
			$newpath   = $fileroot . '/' . $filename . '.' . $fileext;

			$content  = file_get_contents( $imgurl );
			$fp 		  = fopen( $newpath, "w" );
			$result   = fwrite( $fp, $content );
			fclose( $fp );

			if ( $result ) {
				return $newpath;
			}
			return false;
		}

		//}}
		//{{ _createMetaImageCompress

		/**
		 * Create meta data for image compressed
		 *
		 * @param  int    $ID
		 * @param  string $filepath
		 * @param  array  $oldmeta
		 * @param  array  $apidata
		 * @return string
		 */
		protected function _createMetaImageCompress( $ID, $filepath, $oldmeta, $apidata )
		{
			//
			//First delete all old images
			if ( ! $this->_deleteOldImages( $ID, $oldmeta ) ) {
				return false;
			}

			//
			//Create attachment metadata
			$meta = wp_generate_attachment_metadata( $ID, $filepath );
			if ( ! $meta ) {
				$meta = $oldmeta;
			}
			else
			{
				$meta['compressed'] = 1;
				$meta['compressed_LosslessURL']  = $apidata['LosslessURL'];
				$meta['compressed_LossyURL'] 		 = $apidata['LossyURL'];
				$meta['compressed_OriginalSize'] = $apidata['OriginalSize'];
				$meta['compressed_LoselessSize'] = $apidata['LoselessSize'];
				$meta['compressed_LossySize'] 	 = $apidata['LossySize'];
				$meta['compressed_PercentImprovement'] = $apidata['PercentImprovement'];
				
				//Change guid
				$this->_changeCompressedGuid( $ID, $meta );

				//Save stats
				$this->_saveCompressedStats( $ID, $apidata );
			}
			return $meta;
		}

		//}}
		//{{ _deleteOldImages

		/**
		 * Delete all old images
		 *
		 * @param  int    $ID
		 * @param  array  $oldmeta
		 *
		 * @return boolean
		 */
		protected function _deleteOldImages( $ID, $oldmeta )
		{
			global $wpdb;

			$post = get_post( $ID );
			if ( ! $post ) {
				return false;
			}
			$file = get_attached_file( $ID );
			if ( ! $file ) {
				return false;
			}
			if ( ! empty( $oldmeta['thumb'] ) ) 
			{
				// Don't delete the thumb if another attachment uses it.
				if (! $wpdb->get_row( $wpdb->prepare( "SELECT meta_id FROM $wpdb->postmeta WHERE meta_key = '_wp_attachment_metadata' AND meta_value LIKE %s AND post_id <> %d", '%' . $wpdb->esc_like( $oldmeta['thumb'] ) . '%', $ID )) ) {
					$thumbfile = str_replace( basename($file), $oldmeta['thumb'], $file );
					@ unlink( path_join($uploadpath['basedir'], $thumbfile) );
				}
			}

			// Remove intermediate and backup images if there are any.
			if ( isset( $oldmeta['sizes'] ) && is_array( $oldmeta['sizes'] ) ) {
				foreach ( $oldmeta['sizes'] as $size => $sizeinfo ) {
					$intermediate_file = str_replace( basename( $file ), $sizeinfo['file'], $file );
					@ unlink( path_join( $uploadpath['basedir'], $intermediate_file ) );
				}
			}

			$backup_sizes = get_post_meta( $ID, '_wp_attachment_backup_sizes', true );
			if ( is_array($backup_sizes) ) {
				foreach ( $backup_sizes as $size ) {
					$del_file = path_join( dirname($meta['file']), $size['file'] );
					@ unlink( path_join($uploadpath['basedir'], $del_file) );
				}
			}

			wp_delete_file( $file );

			return true;
		}

		//}}
		//{{ _saveCompressedStats

		/**
		 * Delete all old images
		 *
		 * @param  int    $ID
		 * @param  array  $apidata
		 *
		 * @return boolean
		 */
		protected function _saveCompressedStats( $ID, $apidata )
		{
			//Attachment ID's
			$ids = get_option( '__ipoptz_image_compressed_ids' );
			if ( ! $ids || ! is_array( $ids ) ) {
				$ids = array();
			}
			$ids[] = $ID;
			update_option( '__ipoptz_image_compressed_ids', $ids );

			//Compressed count
			$count = get_option( '__ipoptz_image_compressed_count' );
			if ( ! $count ) {
				$count = 0;
			}
			$count++;
			update_option( '__ipoptz_image_compressed_count', $count );

			//Total compressed size
			$oriSize = $apidata['OriginalSize'];
			$total   = get_option( '__ipoptz_image_compressed_total' );
			if ( ! $total ) {
				$total = 0;
			}
			$total += $oriSize;
			update_option( '__ipoptz_image_compressed_total', $total );

			//Total size saved
			$saved = get_option( '__ipoptz_image_compressed_saved' );
			if ( ! $saved ) {
				$saved = 0;
			}
			$totalSaved = 0;
			$lossy = get_option( '__ipoptz_lossy_compression', 'false' );
			if ( $lossy == 'true' ) {
				$totalSaved = $apidata['LossySize'];
			} else {
				$totalSaved = $apidata['LoselessSize'];
			}
			$totalSaved = $oriSize - $totalSaved;
			$saved += $totalSaved;
			update_option( '__ipoptz_image_compressed_saved', $saved );
		}

		//}}
		//{{ _changeCompressedGuid

		/**
		 * Delete all old images
		 *
		 * @param  int    $ID
		 * @param  array  $meta
		 *
		 * @return boolean
		 */
		protected function _changeCompressedGuid( $ID, $meta )
		{
			global $wpdb;
			
			$filepath = $meta['file'];
			$uploads  = wp_upload_dir();
			if ( false === $uploads['error'] )
			{
				$imgurl  = trailingslashit( $uploads['baseurl'] . '/' . _wp_get_attachment_relative_path( $filepath ) ) . basename( $filepath );
				$where = array(
					'ID'		=> $ID
				);
				$updated = array(
					'guid'	=> $imgurl
				);				
				$wpdb->update( $wpdb->posts, $updated, $where );
			}
			
			//
			//Save meta guid
			update_post_meta( $ID, '_wp_attached_file', $filepath );
		}
		//}}
	}
}